/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces;

import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

import javax.xml.namespace.QName;

import org.apache.xerces.dom.DOMMessageFormatter;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.BindingElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.DescriptionElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ImportElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.IncludeElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.InterfaceElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ServiceElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.TypesElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.ApplicationElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.EngineElement;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class DescriptionElementImpl extends AbstractWSDLElement implements DescriptionElement {

    private static final long serialVersionUID = 1L;
    private Set<IncludeElement> includeElements = new HashSet<IncludeElement>();
    private Map<String, Set<ImportElement>> importElements = new LinkedHashMap<String, Set<ImportElement>>();
    private TypesElement typesElement;
    private Map<String, InterfaceElement> interfaceElements = new LinkedHashMap<String, InterfaceElement>();
    private Map<String, BindingElement> bindingElements = new LinkedHashMap<String, BindingElement>();
    private Map<String, ServiceElement> serviceElements = new LinkedHashMap<String, ServiceElement>();
    private Map<String, ApplicationElement> applicationElements = new LinkedHashMap<String, ApplicationElement>();
    private Map<String, EngineElement> engineElements = new LinkedHashMap<String, EngineElement>();

    public DescriptionElementImpl(WSDL20DocumentImpl owner) {
        super(owner, NAME.getNamespaceURI(), NAME.getPrefix() + ":" + NAME.getLocalPart());
    }

    public DescriptionElementImpl() throws DOMException {
        super();
    }

    public void setTargetNamespace(String targetNamespace) {
        setAttribute("targetNamespace", targetNamespace);
    }

    public String getTargetNamespace() {
        return getAttribute("targetNamespace").equals("") ? null : getAttribute("targetNamespace");
    }

    public TypesElement getTypesElement() {
        return typesElement;
    }

    public ImportElement addImportElement(ImportElement importElement) {
        if (importElement.getNamespace() == null) {
            throw new IllegalArgumentException("Required attribute 'namespace' not set.");
        }
        if (includeElements.size() > 0) {
            super.insertBefore(importElement, includeElements.iterator().next());
        } else if (typesElement != null) {
            super.insertBefore(importElement, typesElement);
        } else if (interfaceElements.size() > 0) {
            super.insertBefore(importElement, interfaceElements.values().iterator().next());
        } else if (bindingElements.size() > 0) {
            super.insertBefore(importElement, bindingElements.values().iterator().next());
        } else if (serviceElements.size() > 0) {
            super.insertBefore(importElement, serviceElements.values().iterator().next());
        } else if (applicationElements.size() > 0) {
            super.insertBefore(importElement, applicationElements.values().iterator().next());
        } else if (engineElements.size() > 0) {
            super.insertBefore(importElement, engineElements.values().iterator().next());
        } else {
            super.insertBefore(importElement, null);
        }
        Set<ImportElement> set = importElements.get(importElement.getNamespace());
        if (set == null) {
            set = new HashSet<ImportElement>();
            importElements.put(importElement.getNamespace(), set);
        }
        set.add(importElement);
        return importElement;
    }

    @SuppressWarnings("unchecked")
    public Set<ImportElement> getImportElements(String namespace) {
        Set<ImportElement> set = importElements.get(namespace);
        if (set == null) {
            return Collections.EMPTY_SET;
        }
        return set;
    }

    public IncludeElement addIncludeElement(IncludeElement includeElement) {
        if (includeElement.getLocation() == null) {
            throw new IllegalArgumentException("Required attribute 'location' not set.");
        }
        if (typesElement != null) {
            super.insertBefore(includeElement, typesElement);
        } else if (interfaceElements.size() > 0) {
            super.insertBefore(includeElement, interfaceElements.values().iterator().next());
        } else if (bindingElements.size() > 0) {
            super.insertBefore(includeElement, bindingElements.values().iterator().next());
        } else if (serviceElements.size() > 0) {
            super.insertBefore(includeElement, serviceElements.values().iterator().next());
        } else if (applicationElements.size() > 0) {
            super.insertBefore(includeElement, applicationElements.values().iterator().next());
        } else if (engineElements.size() > 0) {
            super.insertBefore(includeElement, engineElements.values().iterator().next());
        } else {
            super.insertBefore(includeElement, null);
        }
        includeElements.add(includeElement);
        return includeElement;
    }

    public Set<IncludeElement> getIncludeElements() {
        return includeElements;
    }

    public TypesElement setTypesElement(TypesElement typesElement) {
        if (interfaceElements.size() > 0) {
            super.insertBefore(typesElement, interfaceElements.values().iterator().next());
        } else if (bindingElements.size() > 0) {
            super.insertBefore(typesElement, bindingElements.values().iterator().next());
        } else if (serviceElements.size() > 0) {
            super.insertBefore(typesElement, serviceElements.values().iterator().next());
        } else if (applicationElements.size() > 0) {
            super.insertBefore(typesElement, applicationElements.values().iterator().next());
        } else if (engineElements.size() > 0) {
            super.insertBefore(typesElement, engineElements.values().iterator().next());
        } else {
            super.insertBefore(typesElement, null);
        }
        this.typesElement = typesElement;
        return typesElement;
    }

    public InterfaceElement addInterfaceElement(InterfaceElement interfaceElement) {
        if (interfaceElement.getName() == null) {
            throw new IllegalArgumentException("Required attribute 'name' not set.");
        }
        if (bindingElements.size() > 0) {
            super.insertBefore(interfaceElement, bindingElements.values().iterator().next());
        } else if (serviceElements.size() > 0) {
            super.insertBefore(interfaceElement, serviceElements.values().iterator().next());
        } else if (applicationElements.size() > 0) {
            super.insertBefore(interfaceElement, applicationElements.values().iterator().next());
        } else if (engineElements.size() > 0) {
            super.insertBefore(interfaceElement, engineElements.values().iterator().next());
        } else {
            super.insertBefore(interfaceElement, null);
        }
        interfaceElements.put(interfaceElement.getName(), interfaceElement);
        return interfaceElement;
    }

    public Map<String, InterfaceElement> getInterfaceElements() {
        return interfaceElements;
    }

    public BindingElement addBindingElement(BindingElement bindingElement) {
        if (bindingElement.getName() == null) {
            throw new IllegalArgumentException("Required attribute 'name' not set.");
        }
        if (serviceElements.size() > 0) {
            super.insertBefore(bindingElement, serviceElements.values().iterator().next());
        } else if (applicationElements.size() > 0) {
            super.insertBefore(bindingElement, applicationElements.values().iterator().next());
        } else if (engineElements.size() > 0) {
            super.insertBefore(bindingElement, engineElements.values().iterator().next());
        } else {
            super.insertBefore(bindingElement, null);
        }
        bindingElements.put(bindingElement.getName(), bindingElement);
        return bindingElement;
    }

    public Map<String, BindingElement> getBindingElements() {
        return bindingElements;
    }

    public ServiceElement addServiceElement(ServiceElement serviceElement) {
        if (serviceElement.getName() == null) {
            throw new IllegalArgumentException("Required attribute 'name' not set.");
        }
        if (applicationElements.size() > 0) {
            super.insertBefore(serviceElement, applicationElements.values().iterator().next());
        } else if (engineElements.size() > 0) {
            super.insertBefore(serviceElement, engineElements.values().iterator().next());
        } else {
            super.insertBefore(serviceElement, null);
        }
        serviceElements.put(serviceElement.getName(), serviceElement);
        return serviceElement;
    }

    public Map<String, ServiceElement> getServiceElements() {
        return serviceElements;
    }

    public ApplicationElement addApplicationElement(ApplicationElement applicationElement) {
        if (applicationElement.getName() == null) {
            throw new IllegalArgumentException("Required attribute 'name' not set.");
        }
        if (engineElements.size() > 0) {
            super.insertBefore(applicationElement, engineElements.values().iterator().next());
        } else {
            super.insertBefore(applicationElement, null);
        }
        applicationElements.put(applicationElement.getName(), applicationElement);
        return applicationElement;
    }

    public Map<String, ApplicationElement> getApplicationElements() {
        return applicationElements;
    }

    public EngineElement addEngineElement(EngineElement engineElement) {
        if (engineElement.getName() == null) {
            throw new IllegalArgumentException("Required attribute 'name' not set.");
        }
        super.insertBefore(engineElement, null);
        engineElements.put(engineElement.getName(), engineElement);
        return engineElement;
    }

    public Map<String, EngineElement> getEngineElements() {
        return engineElements;
    }

    @Override
    protected Object clone() throws CloneNotSupportedException {
        return super.clone();
    }

    @Override
    public Node appendChild(Node newChild) throws DOMException {
        if (newChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = newChild.getNamespaceURI();
            String localPart = newChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(ImportElement.NAME)) {
                return addImportElement((ImportElement)newChild);
            }
            if (elementName.equals(IncludeElement.NAME)) {
                return addIncludeElement((IncludeElement)newChild);
            }
            if (elementName.equals(TypesElement.NAME)) {
                return setTypesElement((TypesElement)newChild);
            }
            if (elementName.equals(InterfaceElement.NAME)) {
                return addInterfaceElement((InterfaceElement)newChild);
            }
            if (elementName.equals(BindingElement.NAME)) {
                return addBindingElement((BindingElement)newChild);
            }
            if (elementName.equals(ServiceElement.NAME)) {
                return addServiceElement((ServiceElement)newChild);
            }
            if (elementName.equals(ApplicationElement.NAME)) {
                return addApplicationElement((ApplicationElement)newChild);
            }
            if (elementName.equals(EngineElement.NAME)) {
                return addEngineElement((EngineElement)newChild);
            }
        }
        return super.insertBefore(newChild, null);
    }

    @Override
    public Node removeChild(Node oldChild) throws DOMException {
        if (oldChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = oldChild.getNamespaceURI();
            String localPart = oldChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(ImportElement.NAME)) {
                String namespace = ((ImportElement)oldChild).getAttribute("namespace");
                Set<ImportElement> set = importElements.get(namespace);
                if (set != null) {
                    set.remove(oldChild);
                }
            }
            if (elementName.equals(IncludeElement.NAME)) {
                includeElements.remove(oldChild);
            }
            if (elementName.equals(TypesElement.NAME)) {
                typesElement = null;
            }
            if (elementName.equals(InterfaceElement.NAME)) {
                interfaceElements.remove(((Element)oldChild).getAttribute("name"));
            }
            if (elementName.equals(BindingElement.NAME)) {
                bindingElements.remove(((Element)oldChild).getAttribute("name"));
            }
            if (elementName.equals(ServiceElement.NAME)) {
                serviceElements.remove(((Element)oldChild).getAttribute("name"));
            }
            if (elementName.equals(ApplicationElement.NAME)) {
                applicationElements.remove(((Element)oldChild).getAttribute("name"));
            }
            if (elementName.equals(EngineElement.NAME)) {
                engineElements.remove(((Element)oldChild).getAttribute("name"));
            }
        }
        return super.removeChild(oldChild);
    }

    @Override
    public Node replaceChild(Node newChild, Node oldChild) throws DOMException {
        if (oldChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = oldChild.getNamespaceURI();
            String localPart = oldChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(ImportElement.NAME)
                    || elementName.equals(IncludeElement.NAME)
                    || elementName.equals(TypesElement.NAME)
                    || elementName.equals(InterfaceElement.NAME)
                    || elementName.equals(BindingElement.NAME)
                    || elementName.equals(ServiceElement.NAME)
                    || elementName.equals(ApplicationElement.NAME)
                    || elementName.equals(EngineElement.NAME)) {
                throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR, DOMMessageFormatter.formatMessage(
                        DOMMessageFormatter.DOM_DOMAIN, "HIERARCHY_REQUEST_ERR", null));
            }
        }
        return super.replaceChild(newChild, oldChild);
    }

    @Override
    public Node insertBefore(Node newChild, Node refChild) throws DOMException {
        if (newChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = newChild.getNamespaceURI();
            String localPart = newChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(ImportElement.NAME)
                    || elementName.equals(IncludeElement.NAME)
                    || elementName.equals(TypesElement.NAME)
                    || elementName.equals(InterfaceElement.NAME)
                    || elementName.equals(BindingElement.NAME)
                    || elementName.equals(ServiceElement.NAME)
                    || elementName.equals(ApplicationElement.NAME)
                    || elementName.equals(EngineElement.NAME)) {
                throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR, DOMMessageFormatter.formatMessage(
                        DOMMessageFormatter.DOM_DOMAIN, "HIERARCHY_REQUEST_ERR", null));
            }
        }
        return super.insertBefore(newChild, refChild);
    }

}
