/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces.util;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Factory;
import org.w3c.dom.Element;
import org.xml.sax.EntityResolver;

/**
 * Provides information regarding context within which a document is being built.
 * 
 * @author wolfftw
 * 
 */
public interface DocumentContext {

    /**
     * Gets entity resolver.
     * 
     * @return
     */
    public EntityResolver getEntityResolver();

    /**
     * Returns parent element of element currently being parsed. Provides contextual
     * information necessary for creating child elements with same qName, e.g. 'wsdl:fault',
     * 'wsdl:operation', etc ...
     * 
     * @return parent or null if fCurrentElement null or not an element node
     */
    public Element getParent();
    
    /**
     * Returns name of parent element of element currently being parsed. Provides contextual
     * information necessary for creating child elements with same qName, e.g. 'wsdl:fault',
     * 'wsdl:operation', etc ...
     * 
     * @return parent name or null if fCurrentElement null or not an element node
     */
    public QName getParentName();
    
    
    /**
     * Returns factory.
     * 
     * @return factory.
     */
    public WSDL20Factory getFactory();


}
